# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::VMwareCloudSimple::Mgmt::V2019_04_01
  #
  # Description of the new service
  #
  class VirtualNetworks
    include MsRestAzure

    #
    # Creates and initializes a new instance of the VirtualNetworks class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [VMwareCloudSimpleClient] reference to the VMwareCloudSimpleClient
    attr_reader :client

    #
    # Implements list available virtual networks within a subscription method
    #
    # Return list of virtual networks in location for private cloud
    #
    # @param region_id [String] The region Id (westus, eastus)
    # @param pc_name [String] The private cloud name
    # @param resource_pool_name [String] Resource pool used to derive vSphere
    # cluster which contains virtual networks
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<VirtualNetwork>] operation results.
    #
    def list(region_id, pc_name, resource_pool_name, custom_headers:nil)
      first_page = list_as_lazy(region_id, pc_name, resource_pool_name, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Implements list available virtual networks within a subscription method
    #
    # Return list of virtual networks in location for private cloud
    #
    # @param region_id [String] The region Id (westus, eastus)
    # @param pc_name [String] The private cloud name
    # @param resource_pool_name [String] Resource pool used to derive vSphere
    # cluster which contains virtual networks
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(region_id, pc_name, resource_pool_name, custom_headers:nil)
      list_async(region_id, pc_name, resource_pool_name, custom_headers:custom_headers).value!
    end

    #
    # Implements list available virtual networks within a subscription method
    #
    # Return list of virtual networks in location for private cloud
    #
    # @param region_id [String] The region Id (westus, eastus)
    # @param pc_name [String] The private cloud name
    # @param resource_pool_name [String] Resource pool used to derive vSphere
    # cluster which contains virtual networks
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(region_id, pc_name, resource_pool_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'region_id is nil' if region_id.nil?
      fail ArgumentError, 'pc_name is nil' if pc_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'resource_pool_name is nil' if resource_pool_name.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.VMwareCloudSimple/locations/{regionId}/privateClouds/{pcName}/virtualNetworks'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'regionId' => region_id,'pcName' => pc_name},
          query_params: {'api-version' => @client.api_version,'resourcePoolName' => resource_pool_name},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::VMwareCloudSimple::Mgmt::V2019_04_01::Models::VirtualNetworkListResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Implements virtual network GET method
    #
    # Return virtual network by its name
    #
    # @param region_id [String] The region Id (westus, eastus)
    # @param pc_name [String] The private cloud name
    # @param virtual_network_name [String] virtual network id (vsphereId)
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [VirtualNetwork] operation results.
    #
    def get(region_id, pc_name, virtual_network_name, custom_headers:nil)
      response = get_async(region_id, pc_name, virtual_network_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Implements virtual network GET method
    #
    # Return virtual network by its name
    #
    # @param region_id [String] The region Id (westus, eastus)
    # @param pc_name [String] The private cloud name
    # @param virtual_network_name [String] virtual network id (vsphereId)
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(region_id, pc_name, virtual_network_name, custom_headers:nil)
      get_async(region_id, pc_name, virtual_network_name, custom_headers:custom_headers).value!
    end

    #
    # Implements virtual network GET method
    #
    # Return virtual network by its name
    #
    # @param region_id [String] The region Id (westus, eastus)
    # @param pc_name [String] The private cloud name
    # @param virtual_network_name [String] virtual network id (vsphereId)
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(region_id, pc_name, virtual_network_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'region_id is nil' if region_id.nil?
      fail ArgumentError, 'pc_name is nil' if pc_name.nil?
      fail ArgumentError, 'virtual_network_name is nil' if virtual_network_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.VMwareCloudSimple/locations/{regionId}/privateClouds/{pcName}/virtualNetworks/{virtualNetworkName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'regionId' => region_id,'pcName' => pc_name,'virtualNetworkName' => virtual_network_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::VMwareCloudSimple::Mgmt::V2019_04_01::Models::VirtualNetwork.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Implements list available virtual networks within a subscription method
    #
    # Return list of virtual networks in location for private cloud
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [VirtualNetworkListResponse] operation results.
    #
    def list_next(next_page_link, custom_headers:nil)
      response = list_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Implements list available virtual networks within a subscription method
    #
    # Return list of virtual networks in location for private cloud
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_next_with_http_info(next_page_link, custom_headers:nil)
      list_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Implements list available virtual networks within a subscription method
    #
    # Return list of virtual networks in location for private cloud
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::VMwareCloudSimple::Mgmt::V2019_04_01::Models::VirtualNetworkListResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Implements list available virtual networks within a subscription method
    #
    # Return list of virtual networks in location for private cloud
    #
    # @param region_id [String] The region Id (westus, eastus)
    # @param pc_name [String] The private cloud name
    # @param resource_pool_name [String] Resource pool used to derive vSphere
    # cluster which contains virtual networks
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [VirtualNetworkListResponse] which provide lazy access to pages of
    # the response.
    #
    def list_as_lazy(region_id, pc_name, resource_pool_name, custom_headers:nil)
      response = list_async(region_id, pc_name, resource_pool_name, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
